;********************************************************************
;
; BIOS Version 6V23-2.03 TimeStamp 29.07.2023 12:55
;
; entwickelt aus:
; MOPPEL-BIOS fr CP/M Rel. 2.2  V 12.5    30.10.85
; (C) Rheinhard Gssler 85
;
; oprimiert fuer:
; Assembler 8085-Simulator-ID V4.93 Oshonsoft (Vladimir Soso)
; Editor 	Notepad++ TAB=6
;
;
; Stand:
;	29.07.2023	IDE Karte als LW D
;			LW C mit HMS-Format 12.6
;	23.02.2023	V24-Karten nun OK
;	20.02.2023	ohne Multi-IO Karte
;	16.02.2023	2. 87er V24 eingebaut (Console)
;			Softsystem auf 0F700h geschoben
;	11.02.2023	aufgeraeumt, 16Bit TrackNr. funktioniert
;	07.02.2023	Zugriff auf IDE-CF OK
;	05.02.2023	Track als 16Bit Varaible vorbereitet
;	03.02.2023  IDE wird als C: Laufwerk erkannt
;			Speichern nicht moeglich
;			Zieldatei wird nicht geschlossen !
;
;	29.01.2023	WARM-Boot darf nur 22Sektoren lesen
;			jetzt OK; IDE noch ohne Funktion
;	29.01.2023	Test LW A,D ohne IDE
;	24.01.2023	cf durch IDE ersetzt
;	23.01.2023	cf als LW C
;
;	25.04.2022  Tabelle IO-Byte angepasst
;	20.04.2022	87er V24 aingebunden
;	19.04.2022  Zurucke auf 8085 Simulator IDE
;			(io.mac integriert)
;	19.04.2022  Workaround auf M80/L80 YAZE angepasst
;	13.05.2019	DIP-Schalter auf Multi-IO-Karte lesen und als IO-Byte speichern
;	08.05.2019  Zuordnung CO-Routinen neu verteilt (tty, com1, com2, cv24)
;	11.11.2018	DPB 40Tr. DiskSize berichtigt				OK
;	28.10.2018	DPB fuer 40Track Laufwerk		#V18.13
;	20.02.2018	Multi-IO Karte eingebaut		#V18.12	OK
;			IO-Byte ueber DIP-Schalter einstellbar
;	17.02.2018	Quelltext auf TASM umgestellt		#V18.1	OK
;			notepad++ auf Tab=6
;
; Hardware: Moppel-5
;
;	CPU-Karte
;	Video-Karte
;	RAM-Karte 1,2	64kByte
;	FDC-Karte		1LW 3,5"
;	IDE-Karte		1LW mit 8MB
;	1.87er V24		Console / DFUe
;	[2.87er V24]	optional fuer DFUe
;
;
;********************************************************************
;
; Belegung 	IO-Adressen
; fuer Monitorversionen x.5
;
; Stand:	19.02.2022 - Workflow geaendert
;			 	 Assembler M80 unter Yaze
;
;		28.01.2018
;
;--------------------------------------------------------------------
;
; 1. 87er V24-Schnittstellenkarte = COM0
; als Console anstelle der Moppeltastatur und Bildschirmals
;
; ! Adressbereich identisch mit Paralellschnittstelle :
; ! 88,98,a8,b8,c8,d8,e7,f8h
;
; ! Karte modiviziert /DCD dauerhaft auf GND
;
com1s		equ	0e8h		; ACIA 6850 Control/Statusregister
com1d		equ	0f8h		;           Daten lesen/schreiben
;
; Mode-Funktionen
; Auswahl der Schnittstellen
;
;	Bit7 6 5 4 3 2 1 0
;	   : : : : : : : :
;        : : : : : : : + - /Relais 0 (0 = Relais aktiv)
;        : : : : : : + - - /Relais 1 (0 = Relais aktiv)
;        : : : : : + - - - Mod 2 = (20mA) (1= Ausgang aktiv)
;        : : : : + - - - - Mod 3 = (v24)
;	   : : : + - - - - - Mod 4 = (DTR)
;	   : : + - - - - - - Mod 5 = (PRT)
;	   0 0 - - - - - - - = 20mA
;	   0 1 - - - - - - - = V24
;	   1 0 - - - - - - - = PRT
;        1 1 - - - - - - - = Kassette
;
;
;	z.B. 	Mod 4Bh an cs_modex	= V24
;		Mod 43h an cs_modex	= V24 Printer an rckwertigen Buchse
;
cs_mode1	equ	0a8h		; Modus (Eingangswahlschalter)
cs_bell1	equ	0b8h		; Bell  (Schnarre)
;
timer_1s	equ	0d8h		; Timer 8253 Controlregister
timer_10	equ	88h		;        Zaehler 0
timer_11	equ	98h		;	   Zaehler 1 * nicht benutzt
timer_12	equ	0c8h		;	   Zaehler 2 * nicht benutzt
;
startbd	equ	0020		; Startbaudrate 9600Bd
;
;--------------------------------------------------------------------
;
; 2. 87er V24-Schnittstellenkarte = COM1
; als allgemeine DFUe- Schnittstelle
;
; ! Karte modiviziert /DCD dauerhaft auf GND
;
; ACIA 6850
;
com2s		equ	0e9h		; ACIA 6850 Control/Statusregister
com2d		equ	0f9h		;           Daten lesen/schreiben
;
;
; Modus Wahlschalter
;
cs_mode2	equ	0a9h		; Modus (Eingangswahlschalter)
cs_bell2	equ	0b9h		; Bell  (Schnarre)
;
; Timer 8253
;
timer_2s	equ	0d9h		; Timer 8253 Controlregister
timer_20	equ	89h		;        Zaehler 0
timer_21	equ	99h		;	   Zaehler 1 * nicht benutzt
timer_22	equ	0c9h		;	   Zaehler 2 * nicht benutzt
;
;

;--------------------------------------------------------------------
;
; RAM-Bank Umschaltung fuer CP/M Betrieb
;
bank		equ	24h		; Port fuer Bankumschaltung
;
;					; 81h an 24h 0000-3fffh EPROM/VID
;					;            4000-ffffh RAM
;					; 01h an 24h 0000-ffffh RAM
;
;--------------------------------------------------------------------
;
; FDC-Karte
;
fdc_sel	equ	40h		; Floppy Selekt-Byte
fdc_ctl	equ	48h		; WD1720 Controlregister
fdc_sp	equ	49h		; Spurregister
fdc_sek	equ	4ah		; Sektorregister
fdc_dat	equ	4bh		; Datenregister
;
;
;--------------------------------------------------------------------
;
; Parallelschnittstelle (keine ECB-Karte)
;
par_ctl	equ	0d8h		; 8255 Control/Statusregister
par_pa	equ	0a8h		;      Port A
par_pb	equ	0b8h		;      Port B
par_pc	equ	0c8h		;      Port C
;
;
;--------------------------------------------------------------------
;
;
; Echtzeituhr
;
uhr_ctl	equ	19h		; 5832 Controlregister
uhr_dat	equ	18h		;      Datenregister
;
;
;--------------------------------------------------------------------
;
; Thermodrucker
;
therm_d	equ	38h		; Daten
therm_ctl	equ	39h		; Controlregister
;
;
;--------------------------------------------------------------------
;
; Einzelschrittmodul
;
ssm_ff	equ	1ah		; Singlestep-Modul FlipFlop
;
;
;--------------------------------------------------------------------
;
; Videokarte
;
vid_dspen	equ 	2ah		; Format/Blanking
					; (Umschaltung Zeichensatz/
					; Abfrage DiSPlayENable
vidctl	equ	2ah		;
vidreg	equ	28h
vidat		equ	29h
					;
lin		equ	50h		; 80d Zeichen pro zeile
lininv	equ	0ffb0h	; - 80d + 1
;
;
;--------------------------------------------------------------------
;
; HEX-/ASCII-Tastatur)
;
; ASCII-Tastatur			Adressbereich 0ch bis fch
;
ascii		equ	0fch		; oberste Adresse
;					;
;
;--------------------------------------------------------------------
;
;*Ende IO.MAC*******************************************************
;
;
;--------------------------------------------------------------------
;
; Moppel Monitor Routinen
;
ci_m		equ	43h		; Console Input
co_m    	equ	49h		; Console Output
csts_m  	equ	52h		; Console Status
lomo    	equ	4fh		; Lister Output
curson	equ	1036h		; Curson
time		equ	61h		; Uhrzeit lesen und im ASII-Buffer bereitstellen
settim	equ	79h		; Uhrzeit stellen (HL)Std,min, (BC)Wochentag, (DE)Monat,Jahr
;
timbuf	equ	2fe9h		; ASCII-Buffer fuer Uhrzeit
;
ioflag	equ	2fc8h		; LO Select
;
stack0	equ	2950h		; Bank 0 Stack
;
;------------------------------------------------------------------------------
;
;      60-K-Version
;
;
int55		equ	002ch		; IRQ Sprungleiste
int65		equ	0034h		;
int75		equ	003ch		;
;
begin   	equ	0002h   	; bei 60 K
;
bios  	equ	0ea00h	; Start BIOS
ccp   	equ	0d400h	; Start CCP
bdos  	equ	0dc06h	; Start BDOS
;
stack2	equ	0faffh	;
;
verbuf	equ	0fb00h	;
;
buffer	equ	0fc00h	; 1kByte fuer Disk Rd/Wr
nsects	equ	2ch		; Anzahl Sector fr CPML
offset	equ	04h		; Start bei Track 4
vers		equ	22h		; Versions-Nr.
;
buff  	equ	0080h		; default Buffer Adress
stack 	equ	0100h		; default STACK
ndisks	equ	04h		;
iobyte	equ	0003h		;
cdisk 	equ	0004h		; Adr 4 fr last Drive
;
;====================================================================
;
;		aseg

		org	bios
;
       	jmp	boot		; Einsprungleiste
wboote:	jmp	wboot		;
;
; IO-Geraete
;
       	jmp	csts		; Status Console
       	jmp	ci		; Input  Console
       	jmp	co		; Output Console
       	jmp	list		; Output Lineprinter
       	jmp	punch		; Output Puncher
       	jmp	reader	; Input  Reader
;
; Floppy
;
       	jmp	home		;
       	jmp	seldsk	; Select Disk
       	jmp	settrk	; Set    Track
       	jmp	setsec	; Set    Sector
       	jmp	setdma	; Set    Disk-Buffer
       	jmp	read		; Read   Disk *
       	jmp	write		; Write  Disk *
       	jmp	lsts		; Status Lineprinter
       	jmp	sectra	; nicht benutzt
;
; BIOS Erweiterung V24
;
;
; 1. V24 Moppel 87er Hardware	WBOOT + h
;
		jmp	com1_bd	; (DE) Baudrate einstellen
		jmp	com1_ist	; Status (A)=00h kein Zeichen, FFh Zeichen
		jmp	com1_rx	; (A) Byte empfangen, wartet auf Zeichen
		jmp	com1_tx	; (C) Byte senden
;
;
; 2. V24 Moppel 87er Hardware	WBOOT + 30h
;
		jmp	com2_bd	; (DE) Baudrate einstellen
		jmp	com2_ist	; Status (A)=00h kein Zeichen, FFh Zeichen
		jmp	com2_rx	; (A) Byte empfangen, wartet auf Zeichen
		jmp	com2_rxz	; (A) Byte empfangen, abbruch nach 10s mit Control-Z (1ah)
		jmp	com2_tx	; (C) Byte senden
		jmp	com2_txz	; (C) Byte senden, abbruch wenn Geraet nicht bereit
;
;
; BIOS Erweiterung Echtzeituhr MSM 5832
;
		jmp	rdtime	; Uhrzeit auslesen (HL)= zeiger auf ASCII-Buffer
		jmp	wrtime	; Uhrzeit stellen (HL)Std,min, (BC)Wochentag,Kalendertag (DE)Monat,Jahr
;
;
;====================================================================
;
;      DPBase
;
;--------------------------------------------------------------------
;
dpbase:	ds	0h		;
;
; LW A:	80Tr-Laufwerk
dpe0:		dw	xlt0		; Translate table
		dw	0000h
		dw	0000h		; Sratch area
		dw	0000h
		dw	dirbuf	; Dir. Buffer
		dw	dpb0		; Parameter Block
		dw	csv0		; Check
		dw	alv0		; Allocation Vectors
;
; LW B:	80Tr-Laufwek
dpe1:		dw	xlt1
		dw	0000h
		dw	0000h
		dw	0000h
		dw	dirbuf
		dw	dpb0
		dw	csv1
		dw	alv1
;
; LW C:	80Tr-Laufwek	; ss,80Tr.
dpe2:		dw	xlt2
		dw	0000h
		dw	0000h
		dw	0000h
		dw	dirbuf
		dw	dpb0
		dw	csv2
		dw	alv2
;
; LW D:	IDE/CF-Karte
dpe3:		dw	xlt3
		dw	0000h
		dw	0000h
		dw	0000h
		dw	dirbuf
		dw	dpb3
		dw	csv3
		dw	alv3
;
;
;====================================================================
;
;      Disk Parameter Block
;
;--------------------------------------------------------------------
;
; dpb0 fuer 80 Track-Laufwerke
;
dpb0:		dw	20h 		; Sectors per Track
		db	04h 		; Block shift
		db	0Fh 		; Block Mask	BLS = 2048
		db	0h  		; Extnt Mask
		dw	97h 		; Disk Size-1
		dw	3fh 		; Directory max.
		db	0c0h		; Alloc0
		db	00h 		; Alloc1
		dw	10h 		; Check Size
		dw	4h  		; Offset
;
xlt0		equ	0h		;Sec.Translate Vec.;no Transl.Table
;
; dpb1 fuer 80 Track-Laufwerke * wie dpb0 reservirt fuer DS
;
dpb1:		dw   20h 		; Sectors per Track
		db   04h 		; Block shift
		db   0Fh 		; Block Mask	BLS = 2048
		db   0h  		; Extnt Mask
		dw   97h 		; Disk Size-1
		dw   3fh 		; Directory max.
		db   0c0h		; Alloc0
		db   00h 		; Alloc1
		dw   10h 		; Check Size
		dw   4h  		; Offset
;
xlt1   	equ	xlt0
;
; Typ1 aus HMS BIOS V12.6
;
dpb2: 	dw   0020h		; 2x40/1x80 Tracks (2-K-Bloecke)
		db     04h
		db     0fh
		db     01h
		dw   0097h
		dw   007fh
		db    0c0h
		db     00h
		dw   0020h
		dw   0004h

; dpb2a fuer 40 Track-Laufwerke
;	dpb2:		dw	20h		; Sectors per Track
;			db	03h		; Block shift
;			db	07h		; Block Mask	BLS = 1024
;			db	0h 		; Extnt Mask
;			dw	8eh		; Disk Size-1
;			dw	3fh		; Directory max.
;			db	0c0h		; Alloc0
;			db	00h		; Alloc1
;			dw	10h		; Check Size
;			dw	4h 		; Offset
;
xlt2		equ	xlt0
;
; CF-Karte
;
; 8MB File auf CF-Karte
; Offset fr Sektoren sind im AVR-Teil einzurichten
;
; 32 Records a 128Byte
; 4kBloecke
; 512 Dir Eintraege
;     ohne Check (CF-Karte als "Festplatte")
; 4 Systemspuren
;
; Wichtig alv3 anpassen
; Parameter stehen im hinteren Bereich
; ab Soft-System
; alv3 = DSM+1 / 8
;
dpb3:		dw	32  		; Sectors per Track
		db	5  		; Block shift
		db	31  		; Block Mask	BLS = 4096
		db	1   		; Extnt Mask
		dw	2047  	; DSM Disk Size-1 = 8MB
		dw	511  		; Directory max.
		db	11110000b 	; Alloc0
		db	0	  	; Alloc1
		dw	0	  	; Check Size
		dw	1	   	; Offset
;
xlt3		equ	xlt0		; keine Sektorbersetzung
;
;
;====================================================================
;
;      Kalt- und Warmstart
;
;--------------------------------------------------------------------
;
boot:		call	curson	; Cursor einschalten
		mvi	a,01h		;
		out	24h		; Only Bank 1 select
		lxi	sp,stack	; Initialisierung STACK

		call	ideinit	; IDE Interface
		call	com1_ini	; 87er V24 COM1
		call	com2_ini	; 87er V24 COM2
;
		mvi	a,00h		; LPT+CON
		sta	iobyte	; IO-BYTE initialisieren
;
; Systemmeldung ausgeben
;
		lxi	h,msg1	;
		call	string	;

clear:	xra	a		;
		sta	wrtflg	; Buffer ungueltig
		sta	cdisk		; Select Drive 0
		cma			;
		sta	aktdrv	; letztes Drive undefiniert
		lxi	h,0000h	;
		shld	track0	; trk-reg 0+1  undefiniert

gocpm:	mvi	a,0c3h	; Initialisierung Page 0
		sta	0000h		;
		sta	0005h		;
		lxi	h,wboote	; Ziel fuer WBOOT
		shld	1h		;
		lxi	h,bdos	; Ziel fuer BDOS
		shld	6h		;
		lxi	b,buff	;
		call	setdma	;
		lda	cdisk		;
		mov	c,a		;
		jmp	ccp		; ==>> zum CCP

;
; Boot von Floppy
; CCP + BDOS nachladen
;
wboot: 	lxi  	sp,buff	;
		lda  	wrtflg   	; wurde alter Buffer auf Disk
		ora  	a        	; geschrieben ?
		jz  	load
		call	puttrk   	; write alten Buffer auf Disk
;
load:  	xra  	a
		cma
		sta 	wrtflg   	; Buffer ungueltig
		mvi  	c,16h    	; 16h Sektoren nur CCP + BDOS
		lxi  	d,begin  	; Warmstart von Disk
		lxi  	h,ccp		;
bloop: 	push	d
		push	h
		push	b
		mvi	b,18h     	; Select-Byte
		call 	secrd		; Sektor lesen
		pop 	b
		pop 	h
		lxi 	d,0100h	; + Sektorlaenge
		dad 	d
		pop 	d
		ora 	a
		jnz 	load		;
		inr 	e		;
		dcr 	c		;
		jz  	gocpm		; fertig --> CCP
		mov 	a,e
		cpi 	11h		; letzter Sektor vom
		jz  	ntrks		; Track erreicht
		jmp 	bloop
;
ntrks: 	inr 	d		; naechste Spur
		mvi 	e,01h		; 1.Sektor vom Track einstellen
		jmp 	bloop		; noch nicht fertig
;
;
string:	mov	a,m		;
		ora	a		;
		rz			; --> fertig
		mov	c,a		;
		push	h		;
		call	co		;
		pop	h		;
		inx	h		;
		jmp	string	;
;
;
; boot von CF
;
;wboot: 	lxi  	sp,buff
;
;		lda  	wrtflg    	; wurde alter Buffer auf Disk
;		ora	a         	; geschrieben ?
;		jz   	load		; write alten Buffer auf Disk
;		call 	puttrk    	; weiter mit Floppy
;
;load:		xra  a
;		cma
;		sta wrtflg     	; Buffer ungueltig
;;
; CCP und BDOS neu laden
;
;		lxi	h,ccp		; Startadresse 60k CP/M
;		shld	datbuf	; - sichern
;		mvi	a,0h		; Startspur = 0
;		sta	trknr		; - sichern
;		mvi	a,2h		; Startsektor = 2
;		sta	secnr		; - sichern
;
;bootl:	call	settrk	; Spur setzen
;		call	setsec	; Sektor setzen
;		call	rdsek		; Sektor lesen
;		lda	secnr		;
;		inr	a		; Sektornummer hochzaehlen
;		inr	a		; wg. Sektorberechnung im AVR
;		sta	secnr		;
;		cpi	23		; 11 Sektoren laden	!!*
;		jc	bootl		; nein - weiter
;		;
;		jmp	gocpm		; zum ccp
;
;====================================================================
;
;      Peripherie-Verwaltung
;
;	mv24..	= V24 auf Moppel 87er Hardware	9600Bd
;	com1/2	= Multi-IO-Karte
;
;      entfaellt
;     *cv24..	= schnelle V24 auf CF-Karte		115000Bd
;
;--------------------------------------------------------------------
; Console
csts:  	lda	iobyte	; CP/M : Console Status
       	ani	03h
       	lxi	h,csttab
       	jmp	decode
;
ci:    	lda	iobyte	; CP/M : Console Input
       	ani	03h
       	lxi	h,citab	; Startadr CI-Tab
       	jmp	decode
;
co:    	lda	iobyte	; CP/M : Console Output
       	ani	03h
       	lxi	h,cotab	; Startadr CO-Tab
       	jmp	decode
;
; DFUe (Reader)
reader:	lda	iobyte	; CP/M : Reader Input
		rrc
		rrc
		ani	03h
		lxi	h,ritab	; Startadr Reader-Tab
		jmp	decode
;
; DFUe (Puncher)
punch:	lda	iobyte	; CP/M : Puncher Output
		rrc
		rrc
		rrc
		rrc
		ani	03h
		lxi	h,potab	; Startadr Puncher-Tab
		jmp	decode
;
; Drucker (Listgeraet)
list:		lda	iobyte	; CP/M : List Output
		rlc
		rlc
		ani	03h
		lxi	h,lotab	; Startadr List-Tab
		jmp	decode
;
; Druckerstatus
lsts:  	lda	iobyte	; CP/M : Lister Status
       	rlc
       	rlc
       	ani	03h
       	lxi	h,lsttab
       	jmp	decode
;
; DFUe Status
pustat:	lda	iobyte
       	rrc
       	rrc
       	rrc
       	rrc
       	ani	03h
       	lxi	h,pustab
       	jmp	decode
;
; IO-Tabelle auswerten
;
decode:	jz	found		; Setzt PC entsprechend IOBYTE
       	inx	h
       	inx	h
       	dcr	a
       	jmp	decode
found: 	mov	e,m		;Zieladresse auslesen
       	inx	h
       	mov	d,m
       	xchg
       	pchl			;Verzweigung
;
; IO-Tabelle
;
; Console
; -Status
;
csttab:	dw	cs_mon	; TTY	; Moppel Console
       	dw	com1_ist	; CRT	; com1
       	dw	com2_ist	; BAT ; com2
       	dw	com2_ist	; UC1	; com2
;
; -CI
;
citab: 	dw	ci_mon	; TTY	; Moppel Console
       	dw	com1_rx	; CRT	; com1
       	dw	com2_rx	; BAT	; com2
       	dw	com2_rx	; UC1	; com2
;
; -CO
;
cotab: 	dw	co_mon	; TTY	; Moppel Console
       	dw	com1_tx	; CRT	; com1
       	dw	com2_tx	; BAT	; com2
       	dw	com2_tx	; UC1	; com2
;
; List
;
lotab:	dw	co_mon	; TTY	; Moppel Console
		dw	com1_tx	; CRT	; com1
		dw	com2_tx	; LPT	; com2
		dw	com2_tx	; UL1 ; com2
;
; -Status
;
lsttab:	dw	mcosta	; TTY	; Moppel Console
       	dw	com1_ist	; CRT	; com1
       	dw	com2_ist	; LPT   com2
       	dw	com2_ist	; UC1	; com2
;
; Puncher
;
potab:	dw	co_mon	; TTY	; Moppel
		dw	com1_tx	; PTP	; com1
		dw	com2_tx	; UP1	; com2
		dw	com2_tx	; UP2	; com2
;
; -Status
;
pustab:	dw   mcosta		; TTY	; Moppel Console
       	dw   com1_ist	; PTP	; com1
       	dw   com2_ist	; UP1	; com2
       	dw   com2_ist	; UP2	; com2
;
;
; Reader
;
ritab:	dw	ci_mon	; TTY	; Moppel Console
		dw	com2_rx	; PTR	; com1
		dw	com2_rx	; UR1	; com2
		dw	com2_rx	; UR2	; com2
;
;
;====================================================
;
;      Ausgabe ueber die serielle Schnittstelle
;
;====================================================
;
prt:		push	h
		lxi 	h,0h
		dad 	sp		;Bank 1 Stack in H&L
		lxi 	d,stack0
		xchg
		sphl			;Bank 0 Stack
		mvi 	a,81h
		out 	24h   	;Umschalten auf Bank 0

		push	d     	;Bank 1 Stack retten
		mvi 	a,80h 	;Printer waehlen
		sta 	ioflag
		call	lomo  	;Monitor-UP
		mvi 	a,0h  	;Printer inaktiv
		sta 	ioflag	;(wg.CO; sonst doppeltes LO)
		pop 	h     	;Bank 1 Stack zurueck
		mvi 	a,1h
		out 	24h   	;Umschalten auf Bank 1

		sphl	      	;alter Bank 1 Stack
		pop 	h
		ret

prtst:	mvi 	a,0ffh
		ret
;
;
;====================================================
;
;      Ausgabe ueber die parallele Schnittstelle
;
;====================================================
;
centr:	push	h
		lxi 	h,0h
		dad 	sp      	;Bank 1 Stack in H&L
		lxi 	d,stack0
		xchg
		sphl	        	;Bank 0 Stack
		mvi 	a,81h
		out 	24h     	;Umschalten auf Bank 0

		push	d       	;Bank 1 Stack retten
		mvi 	a,40h   	;Centronics waehlen
		sta 	ioflag
		call	lomo    	;Monitor-UP
		mvi 	a,0h    	;Centronics inaktiv
		sta 	ioflag  	;(wg.CO; sonst doppeltes LO)
		pop 	h       	;Bank 1 Stack zurueck
		mvi 	a,1h
		out 	24h     	;Umschalten auf Bank 1

		sphl			;alter Bank 1 Stack
		pop  	h
		ret
;
censta:	mvi  	a,0ffh
       	ret
;
;
;====================================================
;
;     Monitor-Routinen CO, CI und CSTS
;	 schaltet zurueck auf Bank 0
;	 um die Monitorroutinen zu nutzen
;
;====================================================
;
;
;      Ausgabe auf Bildschirm
;
co_mon:	push	h
		lxi 	h,0h
		dad 	sp      	;Bank 1 Stack in H&L
		lxi 	d,stack0
		xchg
		sphl	        	;Bank 0 Stack
		mvi 	a,81h
		out 	24h     	;Umschalten auf Bank 0

		push	d       	;Bank 1 Stack retten
		call	co_m    	;Monitor-UP
		pop 	h       	;Bank 1 Stack zurueck
		mvi 	a,01h
		out 	24h     	;Umschalten auf Bank 1

		sphl	        	;alter Bank 1 Stack
		pop 	h
		ret

mcosta:	mvi  a,0ffh		; Status CO-MON
		ret
;
;
;      Eingabe von der ASCII-Tastatur
;
ci_mon:	push	h
		lxi 	h,0h
		dad 	sp      	;Bank 1 Stack in H&L
		lxi 	d,stack0
		xchg
		sphl	        	;Bank 0 Stack
		mvi 	a,81h
		out 	24h     	;Umschalten auf Bank 0

		push	d       	;Bank 1 Stack retten
		call	ci_m    	;Monitor-UP
		push	psw     	;wg. MOVID-5.3-CSTS
cilop:	call	csts_m  	;Monitor-UP
		jnz 	cilop   	;auf Loslassen warten
		pop 	psw
		mov 	c,a     	;temp. ablegen
		pop 	h       	;Bank 1 Stack zurueck
		mvi 	a,1h
		out 	24h     	;Umschalten auf Bank 1

		sphl	        	;alter Bank 1 Stack
		pop 	h
		mov 	a,c     	;Input zurueckholen
		rlc 	        	;CTL-Codes MOVID 5.3?
		jc  	ctlkey  	;ja: JMP
		mov 	a,c
		ret
;
ctlkey:	mov 	a,c     	;fehlende CTL-Codes erzeugen
		cpi 	8fh     	;HOME?
		jz  	mark    	;ja: JMP
		ani 	7fh
		sui 	20h
		ret
;
mark:		mvi 	a,27h   	;Hochkomma
		ret
;
;
;      Status der ASCII-Tastatur
;
cs_mon:	push	h
		lxi 	h,0h
		dad 	sp      	;Bank 1 Stack in H&L
		lxi 	d,stack0
		xchg
		sphl	        	;Bank 0 Stack
		mvi 	a,81h
		out 	24h     	;Umschalten auf Bank 0

		push	d       	;Bank 1 Stack retten
		call	csts_m  	;Monitor-UP
		pop 	h       	;Bank 1 Stack zurueck
		mov 	c,a     	;Status retten
		mvi 	a,1h
		out 	24h     	;Umschalten auf Bank 1

		sphl	        	;alter Bank 1 Stack
		pop 	h
		mov 	a,c     	;Status zurueckholen
		ret
;
;
;====================================================
;
;      Peripherie-Ansprache (Reserve)
;
;====================================================
;
crtout:	ret
;
crtost:	xra	a
       	ret
;
;
crtin:	ret
;
crtist:	xra	a
       	ret
;
;
com2ost:	xra	a
		ret
;
;
thermo:	ret
;
therst:	xra	a
       	ret
;
resout:	ret
;
rest:  	xra	a
       	ret
;
;
;====================================================
;
;      Disketten-Management
;
;====================================================
;
; BIOS-Funktion Home
;
; I:
; siehe Set Track
;
; BIOS-Funktion Set Track
; 16Bit
;
; I:	(BC)	=	Track
; O:
;
;
home:  	lxi	b,0000h	; Track 0
settrk:	lxi	h,iotrk	;
       	mov	m,c		; Traknummerladen
		inx	h		;
		mov	m,d		;
       	ret
;
;----------------------------------------------------
;
; BIOS-Funktion Set Sektor
; log.Sektor 1-32 wg. Block/deblock
;
; I:	(C)	=	Sektor
;
setsec:	lxi	h,iosec	; Sectorbuffer laden
       	mov	m,c
       	ret
;
;----------------------------------------------------
;
; BIOS-Funktion Set DMA
;
; I:	(BC)	=	DMA Adresse
;
setdma:	mov	l,c		; DMA - Adresse laden
       	mov	h,b
       	shld	iodma
       	ret
;
;----------------------------------------------------
;
;
; BIOS-Funktion Set Drive
;
; I:	(C)	=	Laufwerk
; O:	(HL)	= 	DPH	(HL)=0 Laufwerk nicht vorhanden
;
seldsk:	lxi	h,0000h	; Bei Fehler notwendig
       	mov	a,c
       	cpi	ndisks	; LW im Bereich ?
       	rnc
;
       	sta	iodrv		; LW in Buffer
       	add	a		; ;2
       	add	a		; ;4
       	add	a		; ;8
       	add	a		; ;16 (Laenge eines DPBs)
       	mov	l,a
       	lxi	d,dpbase
       	dad	d
       	ret
;
; BIOS-Funktion Sec Translate
; !im Moppel nicht benoetigt
;
sectra:	mov	a,d
       	ora	e
       	jz	se1		; Test ob Xlate-Tab vorhanden
;
       	xchg
       	dad	b
       	mov	a,m
       	sta	iosec
       	mov	l,a
       	mvi	h,00h
       	ret
;
se1:   	mov	l,c
       	mov	a,c
       	sta	iosec
       	mvi	h,0h
       	ret
;
;
;====================================================

; BIOS-Funktion log.Sektor lesen
;
; I:	vorher alle SET-Befehle Drive, Track, Sektor, DMA
; O:	log.Sektor von Buffer zum BDOS DMA
;	erst wenn Buffer ungueltig ist erfolgt der
;	physikalische Laufwerkszugriff
;
read:  	di			; <== Einsprung von BDOS
		call	calc		;
		call	comp
		jnz 	rload		; DRV, Track, Sek nicht mehr aktuelle
;
r1rd:		call	bufcal	; Daten im Buffer, nur kopieren
		call	copyx
		xra 	a
		ret
;
; bufcal - Record auf 1kByte-Buffer abbilden
;
bufcal:	lxi	h,buffer
       	lda	iosec
       	ani	07h
       	rrc
       	mov	d,a
       	mvi	a,00h
       	rar
       	mov	e,a
       	mov	a,d
       	ani	7fh
       	mov	d,a
       	dad	d
       	xchg
       	lhld	iodma
       	xchg
       	mvi	b,80h		; Recordlaenge = 128 Bytes
       	ret
;
;
; rload - Test ob Buffer gltig ist (wrtflg)=1
;         wenn nein, alten Buffer zurckschreiben
;
rload: 	lda	wrtflg
       	ora	a
       	jz	r1load
       	call	puttrk	; alten Buffer sichern
		jc   	errio
;
; Buffer von Disk holen (1k nchladen)
;
r1load:	call	calc
       	call	store
       	call	gettrk	; 1k lesen --> Buffer
		jc   	errio
       	jmp	r1rd
;
;
; Copyx kopiert die Daten vom Buffer (FC00h) in DMA-Buffer
;			  und umgekehrt (DE-HL) getauscht
;
; I:	(B)	= Recordlaenge
; 	(DE)	= Zieladress
; 	(HL)	= Quelladresse
; O:
;
copyx: 	mov 	a,m		;
		stax	d
		inx 	h
		inx 	d
		dcr 	b
		jnz 	copyx
		ret
;
;----------------------------------------------------
;
; BIOS-Funktion log.Sektor schreiben
;
; I:	vorher alle SET-Befehle Drive, Track, Sektor, DMA
; O:	log.Sektor von BDOS DMA nach Buffer
;	erst wenn Buffer ungueltig ist erfolgt der
;	physikalische Laufwerkszugriff
;
write: 	di			; <== Einsprung von BDOS
		call	calc		;
		call	comp
		jnz 	wload
;
w1wr:		call	bufcal	; Daten im Buffer, nur kopieren
		xchg
		call	copyx		; Record in DMA Buffer
		mvi 	a,01h
		sta 	wrtflg	; Buffer wieder gltig
		xra 	a
		ret
;
;
wload: 	lda	wrtflg	;
		ora 	a
		jz  	w1load
		call	puttrk	; alten Buffer Sichern
		ana	a
		jnz 	errio
;
w1load:	call	calc
		call	store
		call	gettrk	; neuen Buffer holen
		ana 	a
		jnz 	errio
		jmp 	w1wr
;
; Fehlermeldung an BDOS (A)=01h
;
errio:	mvi	a,1h
		ora	a
		ret
;
;====================================================
;
; calc - CP/M Sektorgroee auf Moppel Sektorgroee umrechnen
;
;	CP/M-Format 	32 Sektoren/Track a128Byte [Record]
;	Moppel-Format 	16 Sektoren/Track a256Byte
; I:	iosek		vom BDOS
; O:	(B) 		Laufwerk
;	(C)		Sektor
;	(DE)		Track
;
;	 (E)= Startsektor fr R/W Disk
;
calc:		lda	iosec
		rrc
		ani	0ch
		inr	a
		mov	c,a
		lda	iodrv
		mov	b,a
		lhld	iotrk
		xchg
		ret
;
; comp - Test ob Drive/Track/Sektor noch gleich ist
;
comp:		lda	aktdrv
		cmp	b
		rnz
		lhld	akttrk
		mov	a,l
		cmp	e
		rnz
		mov	a,h
		cmp	d
		rnz
		lda	aktsec
		cmp	c
		ret
;
; store - sicher Drive/Track/Sektor
;	  fr Disk-Routinen
;
store:	mov	a,b		;
		sta	aktdrv	;
		mov	a,c		;
		sta	aktsec
		xchg
		shld	akttrk
		ret
;
;
;----------------------------------------------------
;
;     GETTRK/PUTTRK (Bufferverwaltung)
;	Block lesen/schreiben 1KB-Buffer
;	mit IDE-Interface
;
; I:	datbuf <== iodma BDOS SET-DMA
;	aktdrv <== iodrv BDOS SET-DISK
;	akttrk <== iotrk BDOS SET-Track
;	aktsec <== iosec BDOS SET-Setor
;
;	wrtflg	 	= 0 Buffer ungueltig)
;	trknr	 =	LBA+1	= laufende Spurnummer
;	secnr	 =	LBA	= laufende Sektornummer
;
; O:	Buffer 	= 1KByte Daten, werden dann Recordweise mit DMA ausgetauscht
;	(A)		= 0 fuer OK
;	(CY) 		= 1 Fehler lesen/schreiben
;
;====================================================
;
; Test ob Laufwerk C ausgewaehlt wurde
;			dann zu den CF-Routinen
;
gettrk:	lda	aktdrv	;
		cpi	03h		; Laufwerk = D?
		jnz	get_fl	; nein weiter mit Floppy
		jmp	get_ide	; ==> zu IDE-Routinen

;
puttrk:	lda	aktdrv	;
		cpi	03h		; Laufwerk = D ?
		jnz	put_fl	; nein weiter mit Floppy
		jmp	put_ide	; ==> zu IDE-Routinen

;
; Floppy-Routinen vorbereiten
;	(HL)	Buffer
;	(E)	Sektor
;	(B)	Laufwerksnummer fuer FDC aufbereiten
;	(D)	Anzahl der zu kopierenden Sektoren
;
; codbuf Variable holen und in Register ablegen
;
codbuf:	xra	a		; clear wrtflg -"Buffer leer"
		sta	wrtflg
		lxi	h,buffer   	; Bereitet Reg fuer LW-Anspr.
		lda	aktsec   	;
		mov	e,a
		lda	aktdrv	;
		rlc
		ani	06h
		ori	18h       	; DD / 30ms / Side 0
		mov	b,a
		mvi	d,4h      	; 4 Sectoren
		ret
;
; Buffer mit 4 Sektoren laden
;
get_fl:	call	codbuf    	; Register vorbereiten
;
getlp: 	push	h
		push	d
		push	b
		lda 	akttrk
		mov 	d,a
		call	secrd
		pop 	b
		pop 	d
		pop 	h
		ana 	a
		jnz 	errx
		call	nxtsec
		jnz 	getlp
		xra 	a
		ret			; zurueck zum Disk-Manager
;
;
nxtsec:	push	d         	; Reg fuer naechsten Sec setzten
		lxi	d,0100h
		dad	d         	; Bufferadr + 256 Bytes
		pop	d
		inr	e         	; Sec-Nr. + 1
		dcr	d         	; Sec-Zaehler - 1
		ret
;
; 4 Sektoren vom Buffer auf Floppy schreiben
;
put_fl:	call	codbuf	;
putlp: 	push	h
		push	d
		push	b
		lda 	akttrk
		mov 	d,a
		call	secwr
		pop 	b
		pop 	d
		pop 	h
		ana 	a
		jnz 	errx
		call	nxtsec
		jnz 	putlp
		xra 	a
		ret			; zurueck zum Disk-Manager
;
; Fehler an Diskmanager uebergeben (CY)=1
;
errx:  	stc
		ret
;
;
;====================================================
;
;      Floppy-Routinen
;----------------------------------------------------
; I:		(B)	= Selectbyte		<== Aktdrv
;		(C)	= Anzahl der Sektoren	<== Aktsec
;		(D)	= Start Track 		<== Akttrak
;		(E)	= Start Sektor
;		(HL)	= RAM Ziel/Quelle
; O:		Daten Buffer
;
; secrd
; secwr
;====================================================
;
; IO-Adressen Floppy-Controller
;
comreg      equ	48h		; FDC
trkreg      equ	49h		; FDC
secreg      equ	4ah		; FDC
datreg      equ	4bh		; FDC
status      equ	comreg
ctrl        equ	40h		; (FDC)-Drive-Select
;
;
		nop
;
intrd:	in	datreg
		stax	d
		inx 	d
		ret
;
;
intwr:	ldax	d
		out	datreg	;
		inx	d		;
		ret
;
;
inton:	push	b
		mvi 	a,1eh     	; 8085-RST5.5 enable
		mvi 	b,02h     	; NSC800-RSTC enable
		jmp 	intcte	;
intof:	push	b
		mvi 	a,1fh     	; 8085-RST5.5 disable
		mvi 	b,00h     	; NSC800-RSTC disable
intcte:	sim
		nop 	          	; fuer NSC800-JR-Befehl
		mov 	a,b
		out 	0bbh
		pop 	b
		ret
;
secwrx:	call	init
		push	h
		lxi 	h,intwr
		shld	intprg
		pop 	h
		lda 	trknr
		mov 	d,a
		mvi 	a,2ah
		cmp 	d         	; Sec kleiner Sec 42 ?
		mvi 	a,0a6h
		jnc 	write1
		mov 	b,a
		lda 	selbyt
		ani 	01h
		mov 	a,b
		jnz 	write1
		ani 	0fdh
write1:	sta 	rwbef
		jmp 	start
;
;
;
secrdx:	call	init
		push	h
		lxi 	h,intrd
		shld	intprg
		pop 	h
		mvi 	a,84h		;rdsec;+30 ms
		sta 	rwbef
;
start:	mov	a,m
		rlc
		cnc	recal
		mov	a,m
		ani	7fh
		out	trkreg
		lda	trknr
		out	datreg
		mvi	b,14h		; Seek
		call	posit
		ani	19h		; Verify
		jnz	end0
		in 	trkreg
		mov	d,a
		lda	trknr
		cmp	d
		jnz	poserr
		xra	a
		cmp	b
		mov	a,c
		rz
		in 	trkreg	; FDC-Befehl wurde mit Fehler
		ori	80h		; abgeschlossen
		mov	m,a
		mvi 	b,4h
wdhg:		push	b
		push	h
		call	rdwr
		ani 	5ch
		pop 	h
		pop 	b
		jz  	ende
		dcr 	b
		jnz 	wdhg

end0:		mvi 	m,0h
ende:		in  	status
		ani 	7ch
end1:		push	psw
;
		call	intof
		pop 	psw
		ret
;
recal:	mvi 	b,04h		; Restore
		call	posit
		xra 	a
		cmp 	b
		jz  	recal1
		mvi 	m,80h
		in  	status
		ret
;
recal1:	mov	a,c
		ret
;
;
;
init:		di
		shld	datbuf
		push	b
		mov 	a,d
		sta 	trknr
		mov 	a,e
		sta 	secnr
		;
		; Serviceroutinen vorbereiten
		;
		lxi 	h,0024h
		mvi 	m,0e9h	; TRAP :   PCHL
		lxi 	h,0066h
		mvi 	m,0e9h

		lxi 	h,002ch
		mvi 	m,0f1h	; RST5.5:  POP  PSW
		inx 	h
		mvi 	m,0dbh	;          IN
		inx 	h
		mvi 	m,048h	;          48h
		inx 	h
		mvi 	m,0c9h	;          RET
		;
		; Ende
		;
		mov 	a,b
		sta 	selbyt
		rrc
		ani 	03h
		push	psw
		lxi 	h,laufw0
		mvi 	d,0h
		mov 	e,a
		dad 	d
		mov 	c,m
		lda 	selbyt
		ani 	21h
		ora 	c
		sta 	selout
		lxi 	h,track0
		pop 	psw
		mvi 	d,0h
		mov 	e,a
		dad 	d
		pop 	b
		ret
;
rdwr:		lhld	datbuf
		push	h
		pop 	d
		lhld	intprg
		lda 	secnr
		out 	secreg
		call	inton

		lda 	rwbef
		out 	comreg
rdwr1:	ei
		hlt
		jmp 	rdwr1
;
;
laufw0:	db   02h
laufw1:	db   04h
laufw2:	db   08h
laufw3:	db   10h
;
;
posit:	call	inton
;
		lda	selbyt
		rrc
		rrc
		rrc
		ani	03h
		ora	b
		out	comreg
posit1:	in	status
		rlc
		jnc	posit1
		lda	selout
		out	ctrl
		ei
		mvi	b,2h
posit2:	lxi	d,0ffffh
posit3:	dcx	d
		mov	a,d
		ora	e
		jnz	posit3
		dcr	b
		jnz	posit2
		mvi	a,0d0h
		out	comreg
		call	intof

		in  	status
		ani 	7ch
		ori 	80h
		mov 	c,a
		ret
;
poserr:	mvi	m,0h		;Posit.-Fehler
		mvi	a,10h
		ret
;
;
secrd:	lda	0066h
		sta	nscbuf
;
vsecrd:	shld	hlbuff
;
		lxi 	h,0h
		dad 	sp
		shld	spbuff
		lxi 	sp,stack2
		lhld	hlbuff

		call	secrd0
		shld	hlbuff
		lhld	spbuff
		sphl
		lhld	hlbuff
		push	psw
		lda 	nscbuf
		sta 	0066h
		pop 	psw
		ret
;
;
;
secrd0:	mvi	c,10h
secrdl:	push	h
		push	d
		push	b
		call	secrdx
		pop 	b
		pop 	d
		pop 	h
		ora 	a
		rz
		dcr 	c
		jnz 	secrdl
		inr 	a
		ret
;
secwr:	lda 	0066h
		sta 	nscbuf
		mvi 	c,10h
vsecwr:	nop
secwrl:	push	h
		push	d
		push	b
		call	secwrx
		pop 	b
		pop 	d
		pop 	h
		ora 	a
		jz  	verify
		dcr 	c
		jnz 	secwrl
		inr 	a
		push	psw
		lda 	nscbuf
		sta 	0066h
		pop 	psw
		ret
;
;
verify:	push	h
		shld	vbufhl
;
verxx:	push	d
		push	b
		lxi 	h,verbuf
		call	vsecrd
		lhld	vbufhl
		call	compar
		pop 	b
		pop 	d
		pop 	h
		jnz 	vsecwr
		lda 	nscbuf
		sta 	0066h
		xra 	a
		ret
;
compar:	mvi 	b,0h
		lxi 	d,verbuf
compl:	ldax	d
		cmp 	m
		rnz

		inx 	h
		inx 	d
		dcr 	b
		jnz 	compl
		ret
;
;
;####################################################################
; Modul fuer IDE-Karte
;
; Entwickelt aus IDETOOL von Alfred Klichs 2022,;
; Paul's 8051 Code Library, Paul Stoffregen 2005
; https://www.pjrc.com/tech/8051/ide/
;
;
; Assembler 8085-Simulator-ID V4.93 Oshonsoft (Vladimir Soso)
; Editor 	Notepad++ TAB=6
;
;
; Stand:
;	24.01.2023	aus FDC-0123 vom 23.01.2023
;
;--------------------------------------------------------------------
;
; IO-Adressen fuer PIO 8255
;
PIO_IDE		EQU	058H		; PIO I/O Adresse
;
PIO_LSB		EQU	0 + PIO_IDE ; Port A
PIO_MSB		EQU	1 + PIO_IDE ; Port B
PIO_CTL		EQU	2 + PIO_IDE ; Port C
PIO_CFG		EQU	3 + PIO_IDE ; I/O modus
;
PIO_RDW		EQU	10010010B 	; Port A+B input, Port C output
PIO_WR		EQU	10000000B 	; Port A+B+C output
;
IDE_DRV		EQU	11100000B 	; select the device
			; x1x00000b LBA master
			; x1x10000b LBA slave
			; x0x00000b CHS master
			; x0x10000b CHS slave
;			; Kanal B als Ausgabe
;
;--------------------------------------------------------------------
;
; Variable fuer IDE Funktionen
;
IDE_A0		EQU	00000001B 	; A0 direct to IDE HDD
IDE_A1		EQU	00000010B 	; A1 dto.
IDE_A2		EQU	00000100B 	; A2 dto.
IDE_CS0		EQU	00001000B 	; CS0 to Inverter
IDE_CS1		EQU	00010000B 	; CS1 dto.
IDE_WR		EQU	00100000B 	; WR dto.
IDE_RD		EQU	01000000B 	; RD dto.
IDE_RST		EQU	10000000B 	; RST dto.
;
IDE_DATA		EQU	IDE_CS0
IDE_ERR		EQU	IDE_CS0 + IDE_A0
IDE_SEC_CNT		EQU	IDE_CS0 + IDE_A1
IDE_SECTOR		EQU	IDE_CS0 + IDE_A1 + IDE_A0
IDE_CYL_LSB		EQU	IDE_CS0 + IDE_A2
IDE_CYL_MSB		EQU	IDE_CS0 + IDE_A2 + IDE_A0
IDE_HEAD		EQU	IDE_CS0 + IDE_A2 + IDE_A1
IDE_COMMAND		EQU	IDE_CS0 + IDE_A2 + IDE_A1 + IDE_A0
IDE_STATUS		EQU	IDE_CS0 + IDE_A2 + IDE_A1 + IDE_A0
IDE_CONTROL		EQU	IDE_CS1 + IDE_A2 + IDE_A1
IDE_ASTATUS		EQU	IDE_CS1 + IDE_A2 + IDE_A1 + IDE_A0
;
IDE_CMD_RECAL	EQU	010H
IDE_CMD_READ	EQU	020H
IDE_CMD_WRITE	EQU	030H
IDE_CMD_INIT	EQU	091H
IDE_CMD_ID		EQU	0ECH
IDE_CMD_SPINDN	EQU	0E0H
IDE_CMD_SPINUP	EQU	0E1H
;
;
;===========================================================
;
; IDE/cf-karte lesen/schreiben
; laufwerk 2 = IDE/cf_karte
;
; parameteruebergabe:
;
; trknr		= spur	! Grenze 256Track
; secnr  		= sektor
; datbuf 		= buffer zieladresse
; bufscr (C)	= 2 sektoren
;
;
; CHS_LBA uebertraegt die CHS zu LBA
;
;	LBA	= Sektor
;	LBA+1	= Track lsb
;	LBA+2	= Track msb
;	LBA+3	= Head
;
;
;===========================================================
;
; IDE-karte lesen/schreiben
;  Laufwerk 0 = IDE-Karte
;
; Datenuebergabe:
;
;	akttrk <== iotrk BDOS SET-Track
;	aktsec <== iosec 1.Sektor in Moppelformat
;			 wird im AVR-Teil auf 512B/Sektor umgerechnet
;
;	wrtflg wird auf 0 gesetzt Buffer ungueltig
;
; Parameteruebergabe:
;	datbuf	= aktuelle Bufferadresse
;	trknr		= laufende Spurnummer
;	secnr		= laufende Sektornummer
;
;	LBA	= Sektor
;	LBA+1	= Track lsb
;	LBA+2	= Track msb
;	LBA+3	= Head
;
; Rueckgabe:
;	Buffer 	= 1KByte Daten, werden dann Recordweise mit DMA ausgetauscht
;	(A)		= 0 fuer OK
;	(CY) 		= 1 Fehler lesen/schreiben
;
;--------------------------------------------------------------------
;
; initrw =	Buffer, Spur, Sektorregister vorbereiten
;
; CHS_LBA uebertraegt die CHS zu LBA
;
; Parameteruebergabe:
;
;	datbuf	= aktuelle Bufferadresse
;
;	LBA		= Sektor
;	LBA+1		= Track lsb
;	LBA+2		= Track msb
;	LBA+3		= Head

initrw:	xra  	a		; clear wrtflg -"Buffer ungueltig"
		sta  	wrtflg	;
		di			; Interrupt sperren
		sta	lba+2
		sta	lba+3		; Head 0
		lxi	h,buffer	;
		shld	datbuf	;
		lhld	akttrk	;
		shld	trknr		;
		shld	lba+1		; Track lsb/msb
		lda	aktsec	;
		sta	secnr		;
		sta	lba		; Sektor
		lxi	h,buffer	;
		shld	datbuf	;
		ret			;
;
; get/put_cf =	Block von CF lesen/schreiben
;
get_ide:	call 	initrw	;
		call 	READ_SECTOR	;	;
		ana	a		;
		jnz	cf_err	; Fehler
		;
		call 	updsek	; 2.Sektor laden
		call	READ_SECTOR	;
		ana	a		;
		jnz	cf_err	; Fehler
		xra	a		;
		ret			; zurueck zum Disk-Manager
		;
		;
put_ide:	call 	initrw	;
		call 	WRITE_SECTOR		;
		ana	a		;
		jnz	cf_err	; Fehler
		call 	updsek	;
		call	WRITE_SECTOR		;
		ana	a		;
		jnz	cf_err	; Fehler
		xra	a		;
		ret			; zurueck zum Disk-Manager
;
;
; updsek/sp = Sektor und Spurnummern aktualisieren
;             und an CF-Karte senden
;
;	! maimal 32 Records pro Spur !
;
updsek:	lxi	h,buffer+512
		shld	datbuf
		lda	secnr		; Sektor holen
		cpi	16		; letzter Sektor spur
		jz	updsp		;
		inr	a		; naechster Sektor gleiche Spur
		sta	secnr		;
		sta	lba
		ret

updsp:	mvi	a,01h		; naechste Spur mit Sektor 1
		sta	secnr		;
		sta	lba
		lhld	trknr		;
		inx	h		;
		shld	trknr		;
		shld	lba+1		;
		ret
;
; Fehler (A)=1
;
cf_err:	mvi	a,01		;
		stc			;
		ret			; Zurueck mit Fehler
;
;
;--------------------------------------------------------------------
;
ideinit:	call	IDE_HARD_RESET
		call	IDE_INIT
		jnz	nohd			; IDE Busy
		ret			;

nohd:		lxi	h,nomsg		; keine HD
		call	string		;
		ret

;
IDE_HARD_RESET:
		MVI	A, PIO_RDW		; PIO Read/Write Mode
		OUT	PIO_CFG
		MVI	A, IDE_RST		; hard reset drive
		OUT	PIO_CTL
		PUSH	B
		LXI	B,3000		; ca. 10 ms delay
WLOOP:
		DCX	B
		MOV	A,B
		ORA	C
		JNZ	WLOOP
		POP	B
		XRA	A			; no ide control lines asserted
		OUT	PIO_CTL
		RET
;
; HD Init
;
IDE_INIT:
		call	IDE_GET_ASTATUS	; Test ob Karte
		ana	a			; gesteckt ist
		jz	IDE_NOT_RDY		; nein --> Ende

		MVI	A,IDE_HEAD		;
		MVI	C,IDE_DRV		; select the device
		CALL	IDE_WR_8		;
		LXI	D,0010h		; wait for ready
IDE_RDY_BSY:
		MVI	B,10h
INILP:					; delay for slow drives
		DCR	B
		JNZ	INILP			; djnz $ Z80
		DCX	D
		MOV	A,D
		ORA	E
		JZ	IDE_NOT_RDY		; timeout
		MVI	A,IDE_STATUS
		CALL	IDE_RD_8
		MOV	A,C

		ANI	01000000B		; bit 6,a
						; Z80, if on then ready
		JZ	 IDE_RDY_BSY	; wait for READY bit to be set
		MOV	A,C

		ANI	10000000B		; bit 7,a
						; Z80, if on then busy
		JNZ	 IDE_RDY_BSY	; wait for BUSY bit to be clear
		RET

IDE_NOT_RDY:
		MVI	a,0FFh
		ORA	A
		STC				; timeout
		RET

;
;
; Wait for DRQ
;
IDE_WAIT_DRQ:
		MVI	A,0FFH
		MOV	B,A
		STA	DELAY_DRQ
MORE_DRQ:

		MVI	A,IDE_STATUS
		CALL	IDE_RD_8
		MOV	A,C
		ANI	10001000B
		CPI	00001000B
		JZ	DONE_DRQ
		DCR	B
		JNZ	MORE_DRQ
		LDA	DELAY_DRQ
		DCR	A
		JNZ	MORE_DRQ
		STA	DELAY_DRQ
		STC
		RET
DONE_DRQ:
		XRA	A
		RET

DELAY_DRQ:
		DB	0

;
; Wait for BUSY to be clear
;
IDE_WAIT_NOT_BUSY:
		MVI	A,0FFH
		MOV	B,A
		STA	DELAY_BUSY
MORE_BUSY:
		MVI	A,IDE_STATUS
		CALL	IDE_RD_8
		MOV	A,C
		ANI	11000000B
		XRI	01000000B
		JZ	NOT_BUSY
		DCR	B
		JNZ	MORE_BUSY
		LDA	DELAY_BUSY
		DCR	A
		STA	DELAY_BUSY
		JNZ	MORE_BUSY
		STC
		RET
NOT_BUSY:
		XRA	A
		RET

DELAY_BUSY:
		DB	0

;
; Get Active Status of the IDE Drive
;
IDE_GET_ASTATUS:
		MVI	A,IDE_ASTATUS
		CALL	IDE_RD_8
		MOV	A,C
		RET

;
; 8 bit read
;
IDE_RD_8:
		MOV	D,A			; save adress to D
		OUT	PIO_CTL		; drive address onto control lines
		ORI	IDE_RD		; set read pin on
		OUT	PIO_CTL
;		nop				; delay 2x nop
;		nop
		IN	PIO_LSB		; read lower byte
		MOV	C,A			; to C
		MOV	A,D			; reset read pin first
		OUT	PIO_CTL
		XRA	A			; reset all control pins
		OUT	PIO_CTL
		RET

;
; 8 bit write
;
IDE_WR_8:
		MOV	D,A			; save adress to D
						; Command, head, sector, cylinder etc. in C
		MVI	A, PIO_WR
		OUT	PIO_CFG		; PIO Write Mode
		MOV	A,C
		OUT	PIO_LSB		; Drive lower lines
		MOV	A,D			; Drive address onto control lines
		OUT	PIO_CTL
		ORI	IDE_WR		; set write pin on
		OUT	PIO_CTL
;		nop				; delay	2x nop
;		nop
		MOV	A,D
		OUT	PIO_CTL		; reset write pin first
		XRA	A
		OUT	PIO_CTL		; reset all control pins
		MVI	A,PIO_RDW
		OUT	PIO_CFG		; set PIO read/write mode
		RET

;
; Do the identify command and fill
; the buffer with the drive infos
;
DRIVE_ID:
		CALL	IDE_WAIT_NOT_BUSY
		MVI	A,IDE_HEAD
		MVI	C,IDE_DRV		; select the device
		CALL	IDE_WR_8
		CALL	IDE_WAIT_NOT_BUSY
		MVI	A,IDE_COMMAND
		MVI	C,IDE_CMD_ID	; 0ech
		CALL	IDE_WR_8		; issue the command
		CALL	IDE_WAIT_DRQ
		CALL	READ_DATA
		RET

;
; 16 bit read
;
READ_SECTOR:
						; hdread:
		CALL	IDE_WAIT_NOT_BUSY
		CALL	WR_LBA		; set sector
		MVI	A,IDE_COMMAND	; command register
		MVI	C,IDE_CMD_READ
		CALL	IDE_WR_8
		CALL	IDE_WAIT_DRQ
		CALL	READ_DATA
		CALL	IDE_WAIT_NOT_BUSY
		ANI	10001001B		; busy, DRQ or error
		RZ				; no
		JMP	GET_ERR

;
; 16 bit write
;
WRITE_SECTOR:
						; hdwrite:
		CALL	IDE_WAIT_NOT_BUSY
		CALL	WR_LBA		; set sector
		MVI	A,IDE_COMMAND	; command register
		MVI	C,IDE_CMD_WRITE
		CALL	IDE_WR_8
		CALL	IDE_WAIT_DRQ
		CALL	WRITE_DATA
		CALL	IDE_WAIT_NOT_BUSY
		ANI	10001001B		; busy, DRQ or error
		RZ				; no
						;
						; Return with errors if occurs
GET_ERR:
		MVI	A,IDE_ERR
		CALL	IDE_RD_8
		MOV	A,C
		ORA	A			; set Zero Flag
		RET

;
;
; Read one sector of 512 bytes
;




READ_DATA:	MVI	B,0			; 256 words, 512 bytes per sector
		lhld	datbuf		; target
READ_LP:
		MVI	A,IDE_DATA		; drive address onto control lines
		OUT	PIO_CTL		; and set read pin off (loop)
		ORI	IDE_RD
		OUT	PIO_CTL		; set read pin on
		IN	PIO_LSB		; read the lower byte
		MOV	M,A
		INX	H
		IN	PIO_MSB		; read the upper byte
		MOV	M,A
		INX	H
		DCR	B			; Z80 djnz read_lp
		JNZ	READ_LP
		XRA	A			; set all control pins of
		OUT	PIO_CTL		;
		shld	datbuf		; Zeiger sichern
		ret
;
; Write one sector of 512 bytes
;
WRITE_DATA:	MVI	B,0			; 256 words, 512 bytes per sector
		lhld	datbuf		; target			; source
		MVI	A, PIO_WR
		OUT	PIO_CFG		; PIO Write Mode
WRITE_LP:
		MOV	A,M
		OUT	PIO_LSB		; Drive lower lines with lsb
		INX	H
		MOV	A,M
		OUT	PIO_MSB		; Drive upper lines with msb
		MVI	A,IDE_DATA
		OUT	PIO_CTL		; Drive address onto control lines
		ORI	IDE_WR		; set write pin on
		OUT	PIO_CTL
		MVI	A,IDE_DATA
		OUT	PIO_CTL		; reset write pin
		INX	H
		DCR	B			; Z80 djnz write_lp
		JNZ	WRITE_LP
		XRA	A			; set all control pins of
		OUT	PIO_CTL
		MVI	A,PIO_RDW		; PIO Read/Write Mode
		OUT	PIO_CFG
		shld	datbuf		; Zeiger sichern
		RET
;
;
; LBA = 28bit 4+3x8
; If CHS mode head 8-bit (0-254), sektor 6-bit (1-63)
; cylinder/track 10-bit (0-1023) = 24-bit / 8,38GB max.
;
;
WR_LBA:	LDA	LBA+3			; 4/8-bit, CHS head 0-254
		ANI	0FH			; LBA 0-15
		ORI	IDE_DRV		; mode master/slave CHS/LBA
		MOV	C,A
		MVI	A,IDE_HEAD
		CALL	IDE_WR_8

		LDA	LBA+2			; 8-bit, CHS cyl msb
		MOV	C,A
		MVI	A,IDE_CYL_MSB
		CALL	IDE_WR_8

		LDA	LBA+1			; 8-bit, CHS cyl lsb
		MOV	C,A
		MVI	A,IDE_CYL_LSB
		CALL	IDE_WR_8

		LDA	LBA			; 8-bit, CHS sector

;		and	3fh			; 1-63 if CFC/CHS
		MOV	C,A
		MVI	A,IDE_SECTOR
		CALL	IDE_WR_8

		MVI	C,1			; transfer one sector
		MVI	A,IDE_SEC_CNT
		CALL	IDE_WR_8
		XRA	A
		RET
;
;
;
;--------------------------------------------------------------------
;
;
NOMSG:	DB	"No HDD found!"
		db	0ah		;
		db	0dh		;
		db	00h		;


;
;####################################################################
;
; 1. V24 Moppel 87er Hardware
; Adressen:	88h-f8h
;
; Baudratentabelle fuer
; CPU-Frequenz 6,144MHz
;
; Teilfaktor fuer Timer = CPU-Frequenz /2 = Systemtakt / 4800bd / 16 = 40
;                                                                  I
;                                                                  UART Teiler
; Wert als BCD, da Timer auf BCD Zaehler eingestellt wird
;               sonst Umrechnung nach Binaer notwendig
;
; 9600Bd = 20h lowByte
; 4800Bd = 40h .
; 2400Bd = 80h .
; 1200Bd = 60h lowByte
;	   01h highByte
;
; (DE) Baudrate laden
;
com1_ini:	lxi	d,0020h	; Startwert 9600Bd
;
; Baudrate laden (DE)
;
com1_bd:	mvi	a,37h		; Ctl Zaehler als BCD-Zaehler
		out	timer_1s	;
		mov	a,e		;
		out 	timer_10	; Zaehler untere haelfte
		mov	a,d		;
		out 	timer_10	; Zaehler obere haelfte
		mvi 	a,48h		; v24 modus
		out 	cs_mode1	;
		mvi 	a,03		; Reset UART
		out 	com1s		;
		ret
;
; V24 Input-Status	(A)=00h kein Zeichen
;				(A)=FFh Zeichen vorhanden
;
com1_ist:	mvi 	a,15h		;
		out 	com1s		; clk/16,8bit,1stop,ohne paritaet , RTS off
com1i_l:	in 	com1s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jnz 	com1i_e	; Zeichen vorhanden
		mvi 	a,55h		; RTS on
		out 	com1s	;
		xra 	a		;
		ret			;

com1i_e:	mvi 	a,55h		; RTS on
		out 	com1s	;
		mvi 	a,0ffh	;
		ret			;
;
; Byte von V24 empfangen (A)
; wartet bis Zeichen vorhanden ist
;
com1_rx:	mvi 	a,15h		;
		out 	com1s		; clk/16,8bit,1stop,ohne paritaet , RTS off
com1r_l:	in 	com1s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jz  	com1r_l	; warten
		mvi 	a,55h		; RTS on
		out 	com1s		;
		in 	com1d		; Daten auslesen
		ret			;
;
; Byte v24 senden (C)
; wartet bis Empfaenger bereit ist
;
com1_tx:	mvi 	a,55h		;
		out 	com1s		; clk/16,8bit,1stop,ohne paritaet , RTS on
com1t_l:	in 	com1s		; Warten bis Senderegister leer
		ani 	02h		;
		jz	com1t_l	;
		mov	a,c		;
		out 	com1d		;
		ret			;
;
;
;####################################################################
;
; 2. V24 Moppel 87er Hardware
;
; Baudratentabelle fuer
; CPU-Frequenz 6,144MHz
;
; Teilfaktor fuer Timer = CPU-Frequenz /2 = Systemtakt / 4800bd / 16 = 40
;                                                                  I
;                                                                  UART Teiler
; Wert als BCD, da Timer auf BCD Zaehler eingestellt wird
;               sonst Umrechnung nach Binaer notwendig
;
; 9600Bd = 20h lowByte
; 4800Bd = 40h .
; 2400Bd = 80h .
; 1200Bd = 60h lowByte
;	   01h highByte
;
; (DE) Baudrate laden
;--------------------------------------------------------------------
;
com2_ini:	lxi	d,0020h	; Startwert 9600Bd
;
; Baudrate laden (DE)
;
com2_bd:	mvi	a,37h		; Ctl Zaehler als BCD-Zaehler
		out	timer_2s	;
		mov	a,e		;
		out 	timer_20	; Zaehler untere haelfte
		mov	a,d		;
		out 	timer_20	; Zaehler obere haelfte
		mvi 	a,48h		; v24 modus
		out 	cs_mode2	;
		mvi 	a,03		; Reset UART
		out 	com2s		;
		ret
;
; V24 Input-Status	(A)=00h kein Zeichen
;				(A)=FFh Zeichen vorhanden
;
com2_ist:	mvi 	a,15h		;
		out 	com2s		; clk/16,8bit,1stop,ohne paritaet , RTS off
com2i_l:	in 	com2s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jnz 	com2i_e	; Zeichen vorhanden
		mvi 	a,55h		; RTS on
		out 	com2s			;
		xra 	a		;
		ret			;

com2i_e:	mvi 	a,55h		; RTS on
		out 	com2s	;
		mvi 	a,0ffh	;
		ret			;
;
; Byte von V24 empfangen (A)
; wartet bis Zeichen vorhanden ist
;
com2_rx:	mvi 	a,15h		;
		out 	com2s		; clk/16,8bit,1stop,ohne paritaet , RTS off
com2r_l:	in 	com2s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jz  	com2r_l	; warten
		mvi 	a,55h		; RTS on
		out 	com2s		;
		in 	com2d		; Daten auslesen
		ret			;
;
; Byte v24 senden (C)
; wartet bis Empfaenger bereit ist
;
com2_tx:	mvi 	a,55h		;
		out 	com2s		; clk/16,8bit,1stop,ohne paritaet , RTS on
com2t_l:	in 	com2s		; Warten bis Senderegister leer
		ani 	02h		;
		jz	com2t_l	;
		mov	a,c		;
		out 	com2d		;
		ret			;
;
; Byte V24 empfangen (A)
; mit Zeitueberwachung 10s (A)=1ah Control Z
;
com2_rxz:	push 	b		; ein Byte ueber V24 empfangen (A)
		lxi 	b,10000	; max Wartezeit 10s
		mvi 	a,15h		;
		out 	com2s		; clk/16,8bit,1stop,ohne paritaet , RTS off
com2rz_l:	in 	com2s		; Warten bis Empfangsregister voll
		ani 	01h		;
		jnz  	com2rz_e	;
		call 	dely1		; 1ms warten
		dcr	c		;
		jnz 	com2rz_l	;
		dcr	b		;
		jnz	com2rz_l	;
		pop 	b		;
		mvi	a,55h		; RTS on
		out	com2s	;
		mvi 	a,1ah		; SUB (Control-Z)
		ret

com2rz_e:	mvi	a,55h		; RTS on
		out	com2s		;
		in	com2d		; Daten auslesen
		pop	b		;
		ret			;
;
; Byte V24 senden (C)
; mit Zeitueberwachung
;
com2_txz:	mvi 	a,55h		; ein Byte ueber V24 senden (C)
		out 	com2s		; clk/16,8bit,2stop,ohne paritaet RTS on
		mvi 	b,10		; max Wartezeit 10ms
com2tz_l:	in 	com2s		; Warten bis Senderegister leer
		ani 	02h		;
		jnz  	com2tz_e	;
		call 	dely1		;
		dcr 	b		;
		jnz 	com2tz_l	;
		mvi 	a,04h		; Zeituebschreitung (End of Transmission)
		ret			;
	;
com2tz_e:	mov 	a,c		; Daten ausgeben
		out 	com2d		;
		ret			;
;
;####################################################################
;
; Uhrzeit aus Monitor-Rot holen
; Rueckgabe:
; 	(HL) = Zeiger ASCII-Buffer
;
rdtime:	lxi	h,0h
		dad	sp        	; Bank 1 Stack in H&L
		lxi	d,stack0
		xchg
		sphl           	; Bank 0 Stack
		mvi	a,81h		;
		out	24h		; Bank0 Monitor zuschalten
		push	d         	; Bank 1 Stack retten
		;
		call	time		; Uhrzeit lesen (Monitorprogramm)
		mvi	c,15h		; Byte-Zaehler
		lxi	h,timbuf	; Lesezeiger
		lxi	d,tibuf	; Schreibzeiger
rdtim_l:	mov	a,m		; Daten von Monitor
		stax	d		; nach BIOS-Buffer kopieren
		inx	h		;
		inx	d		; naechste Adresse
		dcr	c		;
		jnz	rdtim_l	; weiter
		mvi	a,24h		; "$"
		stax	d		; Stringabschluss fuer CPM
		;
		pop	h		;Bank 1 Stack zurueck
		mvi	a,1h
		out	24h		;Umschalten auf Bank 1
		sphl			;alter Bank 1 Stack
		;
		lxi	h,tibuf	; Zeiger auf Bufferanfang stellen
		;
		ret
;
; Uhrzeit stellen (HL)Std,min, (BC)Wochentag, Kalendertag (DE)Monat,Jahr
;
wrtime:	ret			; in Arbeit
;
;
;-----------------------------------------------------------------------------
;
; Zeitverzoegerung 1ms, 10ms, 100ms 1sec
;
onesec:	push 	psw		; 1s warten
		mvi 	a,10
one_l:	call 	dely100
		dcr 	a
		jnz 	one_l
		pop 	psw
		ret
;
dely100:	push 	psw		; 100ms warten
		mvi 	a,100
d100_l:	call 	dely1
		dcr 	a
		jnz 	d100_l
		pop 	psw
		ret
;
dely10:	push 	psw		; 10ms warten
		mvi 	a,10
d10_l:	call 	dely1
		dcr 	a
		jnz 	d10_l
		pop 	psw
		ret
;
dely1:	push 	psw		; 1ms warten
		mvi 	a,06h
		add 	a
		add 	a
		inr 	a
		add 	a
		add 	a
		add 	a
d1_l: 	dcr 	a
		jnz 	d1_l
		pop 	psw
		ret
;
;-----------------------------------------------------------------------------
;
; BIOS-Meldetext
;
msg1:  	db	0ah
		db	0dh
		db	"This is 60 K"
		db	" CP/M Rel.2.2"
		db	0ah
		db	0dh
		db	"BIOS IDE-V23.2.04"
		db	" 29.07.2023"
		db	0ah
		db	0dh
		db	00h
;
;=============================================================================
;
; Soft-System
;
		org	0F500h
		;
;
tibuf:	ds   10h		; Buffer fuer Uhrzeit (ASCII)
;
;
;
iodrv: 	db   0h		; <== BDOS
iotrk: 	dw   0h		;	-- " --
iosec: 	db   1h		;	-- " --
iodma: 	dw   buff		;	-- " --
;
wrtflg:	db   0h		; 0= Buffer ungueltig, Daten nachladen
aktdrv:	db   0h		; Laufwerk
akttrk:	dw   0h		; Track nach Blocking/Deblocking
aktsec:	db   0h		; Sektor -- " --
;					; Fuer IDE Interface
lba:		ds	4		; Sektor
;					; Track lsb
;					; Track msb
;					; Head
;
trknr: 	dw   0h		; fuer Disk-Routinen
secnr: 	db   0h		;     -- " --
selbyt:	db   0h		;	-- " --
selout:	db   0h		;	-- " --
;

track0:	db   80h       	; trk-reg undefiniert
track3:	db   80h       	;     --  "  --
;
rwbef: 	db   0h		; fuer Disk-Routinen
datbuf:	dw   0h		; 	-- " --		Bufferadresse
intprg:	dw   0h		; 	-- " --
;
;
nscbuf:	db   0h		; Zwischenspeicher fuer RAM 0066h
;			  		  wg. Floppy-Routine TRAP
hlbuff:	dw   0h		; Zwischenspeicher fuer STACK-Umschaltung
spbuff:	dw   0h		; Zwischenspeicher fuer BDOS-STACK
;
vbufhl:	ds   2h		;
;
;
dirbuf:	ds   80h  		; Directory Access Buffer
alv0:  	ds   12h
csv0:  	ds   10h
alv1:  	ds   12h
csv1:  	ds   10h
alv2:  	ds   12h
csv2:  	ds   10h
alv3:  	ds   256		; fuer 2048 Bloecke a/4096 Byte
csv3:  	ds   10h
enddat:	ds   0h
;
datsiz  	equ	1f6h   	; = ENDDAT - BEGDAT
;
uc1out:	ds   3h        	; User-Consol-Output
uc1in: 	ds   3h        	; User-Consol-Input
uc1sta:	ds   3h        	; User-Consol-Status
;
ptr:		ds   3h        	; Paper-Tape-Reader
ur1:		ds   3h        	; User-Reader 1
ur2:		ds   3h        	; User-Reader 2
;
ptrsta:	ds   3h        	; Paper-Tape-Reader-Status
ur1sta:	ds   3h        	; User-Reader 1 - Status
ur2sta:	ds   3h        	; User-Reader 2 - Status
;
;
;=================================================
;	RAM frei bis
;
;	stack2  	0FAFFh  ; bei 60 K
;
;	verbuf  	0FB00h  ; bei 60 K
;
;	buffer  	0FC00h  ; bei 60 K
;
;
end
;
